package zero1hd.rhythmbullet.audio.analyzer;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.utils.FloatArray;

import edu.emory.mathcs.jtransforms.fft.FloatFFT_1D;
import zero1hd.rhythmbullet.audio.MusicManager;

public class SpectralFluxAnalysisRunnable implements Runnable {
	boolean work = true;
	private boolean done;

	private FloatArray bassSpectralFlux = new FloatArray();
	private FloatArray mSpectralFlux = new FloatArray();
	private FloatArray umSpectralFlux = new FloatArray();

	
	MusicManager musicManager;

	private int PUID;
	private int progress;
	
	public SpectralFluxAnalysisRunnable(MusicManager mm) {
		this.musicManager = mm;
	}
	
	@Override
	public void run() {
		progress = 0;
		int tasksDone = 0;
		long totalTasks = MathUtils.round((float)musicManager.getSampleCount()/musicManager.getChannelCount()/musicManager.getReadWindowSize());
		
		float[] audioPCM = new float[musicManager.getReadWindowSize()];
		float[] spectrum = new float[(musicManager.getReadWindowSize()/2)+1];
		float[] lastSpectrum = new float[(musicManager.getReadWindowSize()/2)+1];
		
		int bassBinBegin = 1;
		int bassBinEnd = 11;
		
		int mBinBegin = 50;
		int mBinEnd = 250;
		
		int umBinBegin = 350;
		int umBinEnd = 513;
		
		Gdx.app.debug("Analyzing Song", musicManager.getBasicSongName());
		Gdx.app.debug("Read freq", String.valueOf(musicManager.getSampleRate()));
		Gdx.app.debug("Using following bin ranges", "\nBass freq begin: " + bassBinBegin + "\nBass freq end: " + bassBinEnd + "\nMain freq begin: " + umBinBegin + "\nMain freq end: " + umBinEnd);

		Gdx.app.debug("Total tasks", String.valueOf(totalTasks));
		
		FloatFFT_1D fft = new FloatFFT_1D(musicManager.getReadWindowSize());
		int seedDigit = 0;
		
		while (musicManager.readSampleFrames(audioPCM) > 0 && work) {
			
			fft.realForward(audioPCM);
			
			//Building a PUID (Pseudo unique ID)
			if (tasksDone == (seedDigit*totalTasks/9)) {
				float avg = 0;
				for (int frame = 0; frame < spectrum.length; frame++) {
					avg += spectrum[frame];
				}
				avg /= spectrum.length;
				if (avg < 0) {
					avg *= -1f;
				}
				PUID +=(int) Math.pow(10, 9-seedDigit) * ((int)(avg*1000f)-(int)(avg*100f)*10);
				seedDigit ++;
			}
			
			System.arraycopy(spectrum, 0, lastSpectrum, 0, spectrum.length);
			System.arraycopy(audioPCM, 0, spectrum, 0, spectrum.length);
			
			float fluxVal;
			//bass detection
			fluxVal = 0;
			for (int i = bassBinBegin; i < bassBinEnd; i++) {
				fluxVal += ((spectrum[i] - lastSpectrum[i])) < 0
						? 0 : (spectrum[i] - lastSpectrum[i]);
			}
			bassSpectralFlux.add(fluxVal);
			
			//m detection
			fluxVal = 0;
			for (int i = mBinBegin; i < mBinEnd; i++) {
				fluxVal += ((spectrum[i] - lastSpectrum[i])) < 0
						? 0 : (spectrum[i] - lastSpectrum[i]);
			}
			mSpectralFlux.add(fluxVal);
			
			//um detection
			fluxVal = 0;
			for (int i = umBinBegin; i < umBinEnd; i++) {
				fluxVal += ((spectrum[i] - lastSpectrum[i])) < 0
						? 0 : (spectrum[i] - lastSpectrum[i]);
			}
			umSpectralFlux.add(fluxVal);
			tasksDone++;
			progress = (int) (100f*tasksDone/totalTasks);
		}
		
		if (work) {
			Gdx.app.debug("Audio Analyzer", "Done getting spectral flux.");
			Gdx.app.debug("Audio Analyzer", "window count: " + bassSpectralFlux.size);
			Gdx.app.debug("Audio Analyzer", "USING SEED: " + PUID);
			progress = 100;
		}
		done = true;
	}
	
	public synchronized int getProgress() {
		return progress;
	}
	public FloatArray getBassSpectralFlux() {
		return bassSpectralFlux;
	}
	public FloatArray getmSpectralFlux() {
		return mSpectralFlux;
	}
	public FloatArray getUmSpectralFlux() {
		return umSpectralFlux;
	}
	public int getPUID() {
		return PUID;
	}
	public boolean isDone() {
		return done;
	}
	public boolean isWorking() {
		return work;
	}
	public void setWork(boolean work) {
		this.work = work;
	}
	public MusicManager getMusicManager() {
		return musicManager;
	}
}
