package zero1hd.rhythmbullet.audio.analyzer;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.utils.FloatArray;

import zero1hd.rhythmbullet.audio.MusicManager;

public class ThresholdCalcRunnable implements Runnable {
	boolean work = true;
	private boolean done;
	
	private MusicManager musicManager;
	
	private FloatArray bassSpectralFlux;
	private FloatArray mSpectralFlux;
	private FloatArray umSpectralFlux;
	private FloatArray bassThreshold = new FloatArray();
	private FloatArray mThreshold = new FloatArray();
	private FloatArray umThreshold = new FloatArray();
	
	public ThresholdCalcRunnable(SpectralFluxAnalysisRunnable sfar) {
		this.bassSpectralFlux = sfar.getBassSpectralFlux();
		this.mSpectralFlux = sfar.getmSpectralFlux();
		this.umSpectralFlux = sfar.getUmSpectralFlux();
		this.musicManager = sfar.getMusicManager();
	}
	
	@Override
	public void run() {
		Gdx.app.debug("Audio Analyzer", "beginning threshold calc.");

		float bassThresholdMultiplier = 1.5f;
		float mThresholdMultiplier = 1.4f;
		float umThresholdMultiplier = 1.4f;
		
		int bassThresholdCalcRange = thresholdRangeCalc(0.27f);
		int mThresholdCalcRange = thresholdRangeCalc(0.4f);
		int umThresholdCalcRange = thresholdRangeCalc(0.4f);
		//threshold calculation
		for (int i = 0; i < umSpectralFlux.size && work; i++) {
			int start = Math.max(0, i - bassThresholdCalcRange/2);
			int end = Math.min(umSpectralFlux.size - 1, i + bassThresholdCalcRange/2);
			float average = 0;
			for (int j = start; j <= end; j++) {
				average += bassSpectralFlux.get(j);
			}
			average /= (end - start);
			bassThreshold.add(average * bassThresholdMultiplier);
			
			start = Math.max(0, i - mThresholdCalcRange/2);
			end = Math.min(umSpectralFlux.size - 1, i + mThresholdCalcRange/2);
			average = 0;
			for (int j = start; j <= end; j++) {
				average+= mSpectralFlux.get(j);
			}
			average /= (end - start);
			mThreshold.add(average*mThresholdMultiplier);
			
			start = Math.max(0, i - umThresholdCalcRange/2);
			end = Math.min(umSpectralFlux.size - 1, i + umThresholdCalcRange/2);
			average = 0;
			for (int j = start; j <= end; j++) {
				average+= umSpectralFlux.get(j);
			}
			average /= (end - start);
			umThreshold.add(average*umThresholdMultiplier);
		}
		Gdx.app.debug("Audio Analyzer", "Threshold calculated.");
		done = true;
	}
	
	private int thresholdRangeCalc(float durationOfRange) {
		return (int) (durationOfRange/(musicManager.getReadWindowSize()/musicManager.getSampleRate()));
	}
	
	public boolean isDone() {
		return done;
	}
	
	public FloatArray getBassSpectralFlux() {
		return bassSpectralFlux;
	}
	public FloatArray getmSpectralFlux() {
		return mSpectralFlux;
	}
	public FloatArray getUmSpectralFlux() {
		return umSpectralFlux;
	}
	public FloatArray getBassThreshold() {
		return bassThreshold;
	}
	public FloatArray getmThreshold() {
		return mThreshold;
	}
	public FloatArray getUmThreshold() {
		return umThreshold;
	}
	
	public MusicManager getMusicManager() {
		return musicManager;
	}
}
