package zero1hd.rhythmbullet.audio.processor;

import com.badlogic.gdx.utils.Disposable;

public interface AudioProcessor extends Disposable {
	/**
	 * Called once, contains the initiation to the stream, only called when play-back begins.
	 * Not thread safe as it should be the first thing to be called during read process.
	 */
	public void initiate();
	
	/**
	 * @return number of channels
	 */
	public boolean isStereo();
	
	/**
	 * @return sample rate
	 */
	public int getSampleRate();
	
	/**
	 * <b>Thread safe</b>
	 * Reads samples (NOT FRAMES) with interwoven data for stereo.
	 * stored in 16 bit format (first 8 are the first byte of data while the second 8 are the second byte of data that composes a short value) 
	 * @param pcm the array the samples should fill
	 * @param syncObj the object that this object should use to synchronize multiple threads.
	 * @return the amount of samples read.
	 */
	public int readSamples(short[] pcm, Object syncObj);
}
