package zero1hd.rhythmbullet.game;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.InputProcessor;
import com.badlogic.gdx.Preferences;
import com.badlogic.gdx.assets.AssetManager;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.utils.Disposable;

import zero1hd.rhythmbullet.controls.KeyMap;
import zero1hd.rhythmbullet.entity.CollisionDetector;
import zero1hd.rhythmbullet.entity.EntityManager;
import zero1hd.rhythmbullet.entity.ally.Laser;
import zero1hd.rhythmbullet.entity.ally.PolyjetEntity;
import zero1hd.rhythmbullet.entity.coordinator.CoordinatorManager;
import zero1hd.rhythmbullet.util.ScoreManager;


public class GameController implements Disposable, InputProcessor {
	public PolyjetEntity polyjet;
	
	public CoordinatorManager cm;
	public EntityManager em;
	private CollisionDetector collisionDetector;
	
	public ScoreManager score = new ScoreManager();
	
	public GameController(AssetManager assetManager, Preferences prefs) {
		Gdx.app.debug("Game Area", "new area created");
		
		polyjet = new PolyjetEntity(assetManager, 25f, 25f, 100, "standard");
		em = new EntityManager(assetManager, prefs);
		cm = new CoordinatorManager(em);
		
		collisionDetector = new CollisionDetector(em.activeEnemies, em.activeAllies, assetManager, prefs);
		em.activeAllies.add(polyjet);
	}
	
	public void draw(Batch batch) {
		batch.begin();
		for (int i = 0; i < em.activeEnemies.size; i++) {
			em.activeEnemies.get(i).draw(batch);
		}
		for (int i = 0; i < em.activeAllies.size; i++) {
			em.activeAllies.get(i).draw(batch);
		}
		batch.end();
	}
	
	public void prepare(float delta) {
		for (int i = 0; i < em.activeEnemies.size; i++) {
			em.activeEnemies.get(i).updatePosition();
		}
		for (int i = 0; i < em.activeAllies.size; i++) {
			em.activeAllies.get(i).updatePosition();
		}
	}
	
	@Override
	public boolean keyDown(int keycode) {
		if (keycode == KeyMap.left) {
			polyjet.moveLeft = true;
		}
		if (keycode == KeyMap.right) {
			polyjet.moveRight = true;
		}
		
		if (keycode == KeyMap.up) {
			polyjet.moveUp = true;
		}
		
		if (keycode == KeyMap.down) {        
			polyjet.moveDown = true;
		}
		
		if (keycode == KeyMap.accelerate) {
			polyjet.accelerate = true;
		}
		return false;
	}
	
	@Override
	public boolean keyUp(int keycode) {
		if (keycode == KeyMap.left) {
			polyjet.moveLeft = false;
		}
		if (keycode == KeyMap.right) {
			polyjet.moveRight = false;
		}
		if (keycode == KeyMap.up) {
			polyjet.moveUp = false;
		}
		if (keycode == KeyMap.down) {
			polyjet.moveDown = false;
		}
		
		if (keycode == KeyMap.accelerate) {
			polyjet.accelerate = false;
		}
		
		if (keycode == KeyMap.shoot) {
			Laser laser	= em.laser.buildEntity();
			laser.init(polyjet.getX() + polyjet.getWidth()/2f, polyjet.getY() + polyjet.getHeight()+1f, 60f);
		}
		return false;
	}

	public PolyjetEntity getPolyjetEntity() {
		return polyjet;
	}
	
	@Override
	public void dispose() {
	}
	
	public CollisionDetector getCollisionDetector() {
		return collisionDetector;
	}

	@Override
	public boolean keyTyped(char character) {
		return false;
	}

	@Override
	public boolean touchDown(int screenX, int screenY, int pointer, int button) {
		return false;
	}

	@Override
	public boolean touchUp(int screenX, int screenY, int pointer, int button) {
		return false;
	}

	@Override
	public boolean touchDragged(int screenX, int screenY, int pointer) {
		return false;
	}

	@Override
	public boolean mouseMoved(int screenX, int screenY) {
		return false;
	}

	@Override
	public boolean scrolled(int amount) {
		return false;
	}
}
