package zero1hd.rhythmbullet.desktop.audio;

import java.util.Observable;
import java.util.Observer;
import java.util.Random;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Preferences;
import com.badlogic.gdx.audio.Music;
import com.badlogic.gdx.audio.Music.OnCompletionListener;

import zero1hd.rhythmbullet.audio.MusicManager;

public class MusicListController extends Observable implements OnCompletionListener, Observer {
	private MusicList musicList;
	private MusicManager mm;
	private int currentPlaybackID;
	private boolean autoPlay;
	private boolean shuffle;
	private Random rand;
	
	private Preferences prefs;
	public MusicListController(MusicList musicList, Preferences prefs) {
		if (prefs == null) throw new NullPointerException("preferences can't be null...");
		if (musicList == null) throw new NullPointerException("music list can't be null...");
		musicList.addObserver(this);
		this.prefs = prefs;
		this.musicList = musicList;
		rand = new Random();
	}
	
	/**
	 * This play method automatically sets the volume.
	 */
	public void play() {
		if (mm != null) {
			Gdx.app.debug("MusicListController", "Playing from MLC.");
			mm.play();
			mm.setVolume(prefs.getFloat("music vol", 1f));
		} else {
			Gdx.app.debug("MusicListController", "failed to begin playing. Load the music!!!");
		}
	}
	
	public void pause() {
		if (mm != null) {
			mm.pause();
		}
	}
	
	public void setMusicByIndex(int index) {
		this.currentPlaybackID = index;
		loadMusic();
	}
	
	public void skip() {
		currentPlaybackID++;
		if (shuffle) {
			shuffle(false);
		}
		loadMusic();
	}
	
	public void previous() {
		currentPlaybackID--;
		if (shuffle) {
			shuffle(false);
		}
		loadMusic();
	}

	@Override
	public void onCompletion(Music music) {
		if (autoPlay) {
			if (shuffle) {
				shuffle(false);
			} else {
				currentPlaybackID++;
			}
			loadMusic();
			play();
		}
	}
	
	/**
	 * Shuffles the controller whether the shuffle boolean is true or false.
	 * @param load whether this method should also make sure to load the music, dispose of the last one, etc. Normally called unless you plan to manually call it elswhere.
	 */
	public void shuffle(boolean load) {
		Gdx.app.debug("MusicListController", "shuffled.");
		if (musicList.getTotal() == 0) {
			currentPlaybackID = 0;
		} else {
			currentPlaybackID = rand.nextInt(musicList.getTotal());
		}
		if (load) {
			loadMusic();
		}
	}
	
	public void setAutoPlay(boolean autoPlay) {
		this.autoPlay = autoPlay;
	}
	
	public void setShuffle(boolean shuffle) {
		this.shuffle = shuffle;
	}
	
	public boolean isShuffle() {
		return shuffle;
	}
	
	public boolean isAutoPlay() {
		return autoPlay;
	}
	
	/**
	 * Loads the current selected song.
	 */
	public void loadMusic() {
		Gdx.app.debug("MusicListController", "music is being loaded and listeners are being notified.");
		if (mm != null) {
			mm.dispose();
		}
		if (currentPlaybackID < 0) {
			currentPlaybackID = musicList.getTotal()-1;
		}
		if (currentPlaybackID >= musicList.getTotal()) {
			currentPlaybackID = 0;
		}
		this.mm = musicList.getMusicManagerFromIndex(currentPlaybackID);
		mm.setOnCompletionListener(this);
		
		setChanged();
		notifyObservers(mm);
	}
	
	public MusicList getMusicList() {
		return musicList;
	}
	
	
	public MusicManager getCurrentMusicManager() {
		return mm;
	}

	@Override
	public void update(Observable o, Object arg) {
		if (o == musicList) {
			loadMusic();
			play();
		}
	}
}
