package zero1hd.rhythmbullet.desktop.audio.visualizer;

import static org.lwjgl.openal.AL10.alGetSourcef;

import java.nio.ByteBuffer;
import java.nio.ShortBuffer;

import org.lwjgl.openal.AL11;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.backends.lwjgl.audio.OpenALMusic;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.utils.reflect.ClassReflection;
import com.badlogic.gdx.utils.reflect.Field;
import com.badlogic.gdx.utils.reflect.ReflectionException;

import zero1hd.rhythmbullet.audio.visualizer.MusicManagerFFT;
import zero1hd.rhythmbullet.audio.visualizer.Visualizer;

public class DesktopVisualizer extends MusicManagerFFT implements Visualizer {
	private ShortBuffer playingBuffer;
	private ShortBuffer compareBuffer;
	private ShortBuffer buffer;
	private int sourceID;

	public DesktopVisualizer() {
		super();
		try {
			Field bufferField = ClassReflection.getDeclaredField(OpenALMusic.class, "tempBuffer");
			bufferField.setAccessible(true);
			buffer = ((ByteBuffer) bufferField.get(null)).asShortBuffer().asReadOnlyBuffer();
		} catch (IllegalArgumentException | SecurityException | ReflectionException e) {
			e.printStackTrace();
			Gdx.app.debug("Visualizer reflection", "Failed attempt at retrieving tempBuffer field.");
			Gdx.app.exit();
		}
	}
	
	/* (non-Javadoc)
	 * @see zero1hd.rhythmbullet.desktop.audio.visualizer.Visualizer#calcPCMData()
	 */
	@Override
	public void calcPCMData() {
		short chanVal;
		int bufferPosOffset = 0;
		bufferPosOffset = updateBufferPosition();
		playingBuffer.position(Math.max(playingBuffer.limit()+bufferPosOffset, 0));

		for (int sid = 0; sid < getAudioPCM().length && sid < playingBuffer.remaining(); sid++) {
			getAudioPCM()[sid] = 0;
			for (int channel = 0; channel < mm.getChannelCount(); channel ++) {
				if (getAudioPCM()[sid] < (chanVal = playingBuffer.get())) {
					getAudioPCM()[sid] = chanVal;
				}
			}
			getAudioPCM()[sid] /= Short.MAX_VALUE+1f;
		}
		
		
		//Take down original buffer position so we don't need to sync again after...
		int originalPos = buffer.position();
		
		//Begin comparison
		buffer.rewind();
		if (compareBuffer.compareTo(buffer) != 0) {
			bufferChanged();
			
			
			//Begin copying current buffer to the comparison buffer
			compareBuffer.clear();
			compareBuffer.put(buffer);
			compareBuffer.flip();
		}
		
		//Reset buffer to proper position.
		buffer.position(originalPos);
	}
	
	private void bufferChanged() {
		//set position to beginning to prepare for overwrite
		playingBuffer.position(0);
		//if the backing playing buffer is full, we have two sections: A and B. A is the one before B, and is read from due to playback latency and offset. B is merely a buffer.
		if (playingBuffer.limit() == playingBuffer.capacity()) {
			//put the second portion into the first...
			playingBuffer.put(playingBuffer.array(), buffer.capacity(), buffer.capacity());
		}
		//put the new buffer into the remaining portion.
		playingBuffer.put(compareBuffer);
	}
	
	private int updateBufferPosition() {
		int pos = (int) ((alGetSourcef(sourceID, AL11.AL_SAMPLE_OFFSET))-buffer.limit()-mm.getChannelCount()*mm.getReadWindowSize());
		return pos;
	}
	
	/* (non-Javadoc)
	 * @see zero1hd.rhythmbullet.desktop.audio.visualizer.Visualizer#setMM(zero1hd.rhythmbullet.audio.MusicManager)
	 */
	@Override
	public void setMM(MusicManager mm) {
		try {
			Field sourceIDField = ClassReflection.getDeclaredField(OpenALMusic.class, "sourceID");
			sourceIDField.setAccessible(true);
			sourceID = (int) sourceIDField.get(mm.getMusic());
		} catch (ReflectionException e) {
			e.printStackTrace();
		}
		int originalPos = buffer.position();

		playingBuffer = ShortBuffer.allocate(buffer.capacity()*2);
		buffer.rewind();
		playingBuffer.put(buffer);
		playingBuffer.flip();
		
		compareBuffer = ShortBuffer.allocate(buffer.capacity());
		buffer.rewind();
		compareBuffer.put(buffer);
		compareBuffer.flip();
		
		buffer.position(originalPos);
		this.mm = mm;
		
		super.setMM(mm);
	}
	
	@Override
	public MusicManager getMM() {
		return mm;
	}
	
	@Override
	public void render(Batch batch, float parentAlpha) {
	}
	
	@Override
	public float[] getAudioPCMData() {
		return getAudioPCM();
	}
	
	@Override
	public void fft() {
		calculate();
	}
}
