package zero1hd.rhythmbullet.desktop.graphics.ui.components;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.scenes.scene2d.ui.Widget;
import com.badlogic.gdx.utils.Disposable;

import zero1hd.rhythmbullet.audio.MusicManager;
import zero1hd.rhythmbullet.audio.visualizer.HorizontalVisualizer;
import zero1hd.rhythmbullet.desktop.audio.visualizer.DesktopVisualizer;

public class HorizontalVisualizerWidget extends Widget implements Disposable {
	private HorizontalVisualizer vis;
	private boolean updatePositioning = true;
	private MusicManager mm;
	private boolean initialLoad;
	private float visRefreshRate;
	private float timer;
	public HorizontalVisualizerWidget() {
		vis = new HorizontalVisualizer(new DesktopVisualizer());
	}
	
	@Override
	public void draw(Batch batch, float parentAlpha) {
		vis.render(batch, parentAlpha);
		super.draw(batch, parentAlpha);
	}
	
	@Override
	public void act(float delta) {
		if (mm != null && mm.isFinishedLoading() && !initialLoad) {
			visRefreshRate = mm.getReadWindowSize()/mm.getSampleRate();
			Gdx.app.debug("Visualizer", "\nsample count: " + mm.getSampleCount()
			+ "\nDuration in seconds: " + mm.getDuration() + 
			"\nSample rate: " + mm.getSampleRate() +
			"\nRefresh rate: " + visRefreshRate);
			vis.setMM(mm);
			initialLoad = true;
		}
		
		if (updatePositioning) {
			vis.updatePositionInfo();
			vis.setX((getWidth() - vis.getActualWidth())/2f);
		}
		
		vis.update(delta);
		updateVisualizerProperties();
		if (mm != null && initialLoad) {
			if (timer >= visRefreshRate) {
				timer = 0;
				vis.calculate();
				vis.calcPCMData();
			} else {
				timer += delta;
			}
		}
		super.act(delta);
	}
	
	public void setMM(MusicManager mm) {
		initialLoad = false;
		this.mm = mm;
	}
	
	@Override
	public void setColor(Color color) {
		vis.setColor(color);
		super.setColor(color);
	}
	
	@Override
	public void setColor(float r, float g, float b, float a) {
		vis.setColor(r, g, b, a);
		super.setColor(r, g, b, a);
	}
	
	public void updateVisualizerProperties() {
		vis.setHeight(getHeight());
		vis.setWidth(getWidth());
		vis.setX(getX());
		vis.setY(getY());
		vis.setRotation(getRotation());
	}
	
	public void setUpdatePositioning(boolean updatePositioning) {
		updateVisualPosition();
		this.updatePositioning = updatePositioning;
	}
	
	public void updateVisualPosition() {
		updateVisualizerProperties();
		vis.updatePositionInfo();
		vis.setX(((vis.getWidth() - vis.getActualWidth())/2f));
		vis.updatePositionInfo();
	}
	
	public boolean isUpdatePositioning() {
		return updatePositioning;
	}
	
	public HorizontalVisualizer getVisualizerType() {
		return vis;
	}

	@Override
	public void dispose() {
		vis.dispose();
	}
	
}
