package zero1hd.rhythmbullet.desktop.graphics.ui.components;

import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.WidgetGroup;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.utils.Disposable;

import zero1hd.rhythmbullet.audio.MusicInfo;
import zero1hd.rhythmbullet.desktop.graphics.ui.pages.MusicSelectionPage;
import zero1hd.rhythmbullet.graphics.ui.components.ShortenedTextLabel;

public class MusicSelectable extends WidgetGroup implements Disposable {
	private Table table;

	private ShortenedTextLabel displayName;

	private boolean selected;
	
	private MusicSelectionPage msp;
	private MusicInfo musicInfo;
	
	private FileHandle musicFile;
	
	public MusicSelectable(FileHandle musicFile, Skin skin, MusicSelectionPage msp) {
		this.musicFile = musicFile;
		table = new Table(skin);
		table.setBackground("holo-pane");
		table.setFillParent(true);
		this.msp = msp;
		setName(musicFile.nameWithoutExtension());
		table.defaults().pad(5f).space(15f).expandX();

		displayName = new ShortenedTextLabel(musicFile.nameWithoutExtension().replace('_', ' '), skin, "sub-font", skin.getColor("default"));
		table.add(displayName);

		table.pack();
		addActor(table);
		addListener(new ClickListener() {
			@Override
			public void clicked(InputEvent event, float x, float y) {
				select();
				super.clicked(event, x, y);
			}
		});
	}
	
	/**
	 * updates the UI side of information.
	 * needs to be called in thread with gl context.
	 * @param musicInfo the music information for this song.
	 */
	public void updateInfo(MusicInfo musicInfo) {
		this.musicInfo = musicInfo;
		displayName.setOriginalText(musicInfo.getMusicName());
	}
	
	
	@Override
	public void layout() {
		displayName.setTargetWidth((int) (getWidth()*0.8f));
		displayName.resize();
		super.layout();
	}
	
	@Override
	public void draw(Batch batch, float parentAlpha) {
		super.draw(batch, parentAlpha);
	}
	
	@Override
	public void act(float delta) {
		super.act(delta);
	}

	public boolean isMusicInvalid() {
		return musicInfo.isInvalidMusic();
	}
	
	/**
	 * Selects this panel
	 */
	public void select() {
		msp.deselectAll();
		table.setBackground("holo-pane-down");
		selected = true;
		msp.setCurrentlySelected(this);
	}
	
	public void deselect() {
		table.setBackground("holo-pane");
		selected = false;
	}
	
	public boolean isSelected() {
		return selected;
	}
	
	@Override
	public float getPrefHeight() {
		return table.getMinHeight();
	}
	
	@Override
	public void dispose() {
	}
	
	public FileHandle getMusicFile() {
		return musicFile;
	}
	
	public MusicInfo getMusicInfo() {
		return musicInfo;
	}
}
