package zero1hd.rhythmbullet.desktop.graphics.ui.components;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.assets.AssetManager;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.Pixmap.Format;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.ParticleEffect;
import com.badlogic.gdx.graphics.g2d.ParticleEffectPool;
import com.badlogic.gdx.graphics.g2d.ParticleEffectPool.PooledEffect;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.utils.Array;
import com.badlogic.gdx.utils.Disposable;

import zero1hd.rhythmbullet.audio.visualizer.MirrorVisualizer;

public class TitleBarVisualizer extends Group implements Disposable {
	private HorizontalVisualizerWidget visual;
	private MirrorVisualizer visual2;
	private Texture bgTexture;
	private Image bg;
	private Image titleImage;
	
	//Particle pools;
	ParticleEffectPool beatEffectPool;
	Array<PooledEffect> effects = new Array<>();
	
	private float particleLimitTime;
	public TitleBarVisualizer(AssetManager assets) {
		if (assets == null) throw new  NullPointerException("TitleBarVisualizer requires assets manager... ITS NULL YOU FOOL");
		visual = new HorizontalVisualizerWidget();
		visual.getVisualizerType().setSpaceBetweenBars(visual.getVisualizerType().getBarWidth()- 2);
		addActor(visual);
		
		setSize(Gdx.graphics.getWidth(), Gdx.graphics.getHeight()*0.2f);
		setPosition(0, (Gdx.graphics.getHeight()-getHeight())/2f);
		visual.setWidth(getWidth());
		visual.setHeight(getHeight());
		visual.getVisualizerType().flip();
		visual.getVisualizerType().reverse();
		visual2 = new MirrorVisualizer();
		visual.setUpdatePositioning(false);
		visual.getVisualizerType().addMirrorVisualizer(visual2);
		visual2.setyPos(MathUtils.round(getHeight()));
		visual.setY(-2);
		visual.updateVisualPosition();
		
		Pixmap pixmap = new Pixmap(MathUtils.round(getWidth()), MathUtils.round(getHeight()), Format.RGBA8888);
		pixmap.setColor(Color.WHITE);
		pixmap.drawLine(0, 0, pixmap.getWidth(), 0);
		pixmap.drawLine(0, 1, pixmap.getWidth(), 1);
		pixmap.drawLine(0, pixmap.getHeight()-1, pixmap.getWidth(), pixmap.getHeight()-1);
		pixmap.drawLine(0, pixmap.getHeight()-2, pixmap.getWidth(), pixmap.getHeight()-2);
		bgTexture = new Texture(pixmap);
		pixmap.dispose();

		bg = new Image(bgTexture);
		bg.setSize(getWidth(), getHeight());
		addActor(bg);
		
		titleImage = new Image(assets.get("title.png", Texture.class));
		titleImage.setScale((bg.getHeight()-(0.1f*Gdx.graphics.getHeight()))/titleImage.getHeight());

		if (titleImage.getWidth() > 0.8f*getWidth()) {
			titleImage.setScale((bg.getWidth()-(0.2f*Gdx.graphics.getWidth()))/titleImage.getWidth());
		}
		titleImage.setPosition((getWidth() - titleImage.getWidth()*titleImage.getScaleX())/2f, (getHeight() - titleImage.getHeight()*titleImage.getScaleY())/2f -5);
		titleImage.setColor(Color.WHITE);
		addActor(titleImage);
		
		beatEffectPool = new ParticleEffectPool(assets.get("beateffect.p", ParticleEffect.class), 0, 8) {
			@Override
			protected PooledEffect newObject() {
				PooledEffect effect = super.newObject();
				effect.scaleEffect(Gdx.graphics.getWidth()/64f);
				return effect;
			}
		};
	}
	
	@Override
	public void act(float delta) {
		if (particleLimitTime > 1f/60f) {
			if (visual.getVisualizerType().getVisualizer().getMM() != null && visual.getVisualizerType().getVisualizer().getMM().isPlaying() && visual.getVisualizerType().getCurrentAvg() > visual.getVisualizerType().getMaxAvgHeight()*0.55f) {
				PooledEffect effect = beatEffectPool.obtain();
				effect.setPosition(0, 0);
				effects.add(effect);
				particleLimitTime = 0;
			}
		} else {
			particleLimitTime += delta;
		}

		super.act(delta);
	}
	
	@Override
	public void draw(Batch batch, float parentAlpha) {
		for (int i = 0; i < effects.size; i++) {
			effects.get(i).draw(batch, Gdx.graphics.getDeltaTime());
			if (effects.get(i).isComplete()) {
				effects.get(i).free();
				effects.removeIndex(i);
			}
		}
		super.draw(batch, parentAlpha);
	}
	
	public HorizontalVisualizerWidget getHvisual() {
		return visual;
	}

	@Override
	public void dispose() {
		visual.dispose();
		bgTexture.dispose();
	}
	
	@Override
	public void setColor(Color color) {
		super.setColor(color);
	}
	
	@Override
	public void setColor(float r, float g, float b, float a) {
		super.setColor(r, g, b, a);
	}
}
