package zero1hd.rhythmbullet.desktop.graphics.ui.pages;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.Slider;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;

import zero1hd.rhythmbullet.RhythmBullet;
import zero1hd.rhythmbullet.audio.MusicInfo;
import zero1hd.rhythmbullet.audio.MusicManager;
import zero1hd.rhythmbullet.audio.analyzer.AudioAnalyzer;
import zero1hd.rhythmbullet.desktop.screens.GameScreen;

public class AnalysisPage extends Page {
	private boolean confirmed;
	private TextButton backButton;
	private AudioAnalyzer aa;
	private Table table;
	private Table adjustment;
	private Label difficultyModLabel, healthModLabel, speedModLabel;
	private Slider difficultyModifierSlider, healthModifierSlider, speedModifierSlider;
	private Label diffModPercentLabel, heltModPercentLabel, speeModPercentLabel;
	private Label progressLabel;
	private TextButton confirmButton;
	private Image albumArt;
	private RhythmBullet core;
	
	public AnalysisPage(RhythmBullet core, Vector3 cameraPosition) {
		table = new Table();
		table.setFillParent(true);
		table.defaults().space(10f);
		addActor(table);
		this.core = core;
		adjustment = new Table();
		Skin skin = core.getDefaultSkin();
		
		difficultyModLabel = new Label("Difficulty Modifier: ", skin, "sub-font", skin.getColor("default"));
		difficultyModifierSlider = new Slider(1, 3, 0.5f, false, skin);
		diffModPercentLabel = new Label(String.valueOf(difficultyModifierSlider.getValue()) + "x", skin);
		difficultyModifierSlider.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				diffModPercentLabel.setText(String.valueOf(difficultyModifierSlider.getValue()) + "x");
			}
		});
		adjustment.add(difficultyModLabel);
		adjustment.add(difficultyModifierSlider).minWidth(0.5f*getWidth());
		adjustment.add(diffModPercentLabel).spaceLeft(10f).center().expandX().fill();
		adjustment.row();
		healthModLabel = new Label("Health Modifier: ", skin, "sub-font", skin.getColor("default"));
		healthModifierSlider = new Slider(1f, 3f, 0.5f, false, skin);
		heltModPercentLabel = new Label(String.valueOf(healthModifierSlider.getValue()) + "x", skin);
		healthModifierSlider.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				heltModPercentLabel.setText(String.valueOf(healthModifierSlider.getValue()) + "x");
			}
		});
		adjustment.add(healthModLabel);
		adjustment.add(healthModifierSlider).fillX();
		adjustment.add(heltModPercentLabel).spaceLeft(10f);
		adjustment.row();
		speedModLabel = new Label("Speed Modifier: ", skin, "sub-font", skin.getColor("default"));
		speedModifierSlider = new Slider(1, 3, 0.5f, false, skin);
		speeModPercentLabel = new Label(String.valueOf(speedModifierSlider.getValue()) + "x", skin);
		speedModifierSlider.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				speeModPercentLabel.setText(String.valueOf(speedModifierSlider.getValue()) + "x");
			}
		});
		adjustment.add(speedModLabel);
		adjustment.add(speedModifierSlider).fillX();
		adjustment.add(speeModPercentLabel).spaceLeft(10f);
		adjustment.row();
		
		confirmButton = new TextButton("Confirm", skin);
		confirmButton.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				confirmed = true;
				confirmButton.setDisabled(true);
				speedModifierSlider.setDisabled(true);
				healthModifierSlider.setDisabled(true);
				difficultyModifierSlider.setDisabled(true);
				
				progressLabel.setText("Loading...");
			}
		});
		
		adjustment.add(confirmButton).colspan(3).fillX();
		adjustment.row();
		progressLabel = new Label("Please confirm... ", skin);
		adjustment.add(progressLabel).colspan(2).left().spaceTop(20f);
		
		backButton = new TextButton("Cancel", skin);
		backButton.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				cameraPosition.x = 1.5f*Gdx.graphics.getWidth();
				aa.dispose();
			}
		});
		backButton.setPosition(15, getHeight()-backButton.getHeight()-25);
		addActor(backButton);
	}
	
	public void processSong(MusicManager mm, Texture albumCover, MusicInfo mi) {
		this.albumArt = new Image(albumCover);
		confirmButton.setDisabled(false);
		speedModifierSlider.setDisabled(false);
		healthModifierSlider.setDisabled(false);
		difficultyModifierSlider.setDisabled(false);
		
		table.clear();
		table.add(this.albumArt).size(adjustment.getMinHeight());
		table.row();
		table.add(adjustment);
		if (aa != null) {
			aa.dispose();
		}
		
		aa = new AudioAnalyzer(mm);
		aa.start();
	}

	@Override
	public void act(float delta) {
		if (aa != null && aa.isDone()) {
			if (confirmed) {
				core.setScreen(new GameScreen(core.getAssetManager(), core.getPrefs()));
			}
 		}
		super.act(delta);
	}
	
	@Override
	public void dispose() {
		aa.dispose();
		super.dispose();
	}
}
