package zero1hd.rhythmbullet.desktop.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.ScreenAdapter;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.utils.viewport.ScreenViewport;

import zero1hd.rhythmbullet.RhythmBullet;
import zero1hd.rhythmbullet.util.AdvancedResizeScreen;

public class SplashScreen extends ScreenAdapter implements AdvancedResizeScreen {
	private Stage stage;
	private RhythmBullet core;
	private Texture splash;
	private Image zero1HD;
	private boolean done;
	
	public SplashScreen(RhythmBullet core) {
		this.core = core;
	}
	
	@Override
	public void show() {
		super.show();
	}
	
	@Override
	public void render(float delta) {
		Gdx.gl.glClearColor(1f, 1f, 1f, 1f);
		Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
		stage.act(delta);
		
		if (!zero1HD.hasActions() && core.isInitComplete()) {
			attemptMoveOn();
		}
		
		stage.draw();
		super.render(delta);
	}
	
	private void attemptMoveOn() {
		if (!done) {
			Gdx.app.debug("Loading Screen", "queue has all been loaded. Action is done playing.");
			done = true;
			core.setScreen(new MainMenuScreen(core));
			zero1HD.remove();
		}
	}
	
	@Override
	public void hide() {
		core.setInitComplete();
		splash.dispose();
		super.hide();
	}

	@Override
	public void preAssetLoad() {
		if (stage != null) {
			stage.dispose();
			splash.dispose();
		}
	}

	@Override
	public void postAssetLoad() {
		stage = new Stage(new ScreenViewport());
		splash = new Texture(Gdx.files.internal("splashlogo.png"));
		zero1HD = new Image(splash);
		zero1HD.setScale((Gdx.graphics.getHeight()*0.8f)/zero1HD.getHeight());
		
		zero1HD.setColor(0f,1f,1f,0f);
		stage.addActor(zero1HD);
		zero1HD.setPosition((stage.getWidth() - zero1HD.getWidth()*zero1HD.getScaleX())/2f, (stage.getHeight() - zero1HD.getHeight()*zero1HD.getScaleY())/2f);
		zero1HD.addAction(Actions.sequence(Actions.color(Color.WHITE, 1f), Actions.fadeOut(0.5f)));
	}
}
